@@ -1,5 +1,4 @@
 import sys
-import traceback
 import aiohttp
 import asyncio
 import acapelladb
@@ -76,8 +75,15 @@ def with_retries(partition_key: List[str] = None, clustering_key: Optional[List[
                         with_retries_logger.warn(f'partition {partition} | clustering {clustering}: timeout')
 
                 except (*aiohttp_errors,
-                        asyncio.TimeoutError) as e:
-                    log_aiohttp_error(e, with_retries_logger)
+                        asyncio.TimeoutError,
+                        acapelladb.utils.errors.KvError) as e:
+                    if isinstance(e, acapelladb.utils.errors.KvError):
+                        if with_retries_logger:
+                            with_retries_logger.exception(e)
+                        if type(e) is not acapelladb.utils.errors.KvError:
+                            break
+                    else:
+                        log_aiohttp_error(e, with_retries_logger)
                     retry_count += 1
 
                     if retry_count == retries_max:
@@ -89,11 +95,10 @@ def with_retries(partition_key: List[str] = None, clustering_key: Optional[List[
                     backoff_timeout *= backoff
                     if with_retries_logger:
                         with_retries_logger.warn(f'backoff timeout increase, now: {backoff_timeout} sec')
-                except Exception as e:
+                except Exception:
                     if with_retries_logger:
-                        with_retries_logger.error(''.join(traceback.format_stack()))
-                    if type(e) is not acapelladb.utils.errors.KvError:
-                        break
+                        with_retries_logger.exception(e)
+                    break
                 else:
                     retry_count = 1
 
@@ -117,10 +122,17 @@ class Batch:
     def batch(self):
         return self.__batch
 
-    async def send(self):
-        @with_retries(success_max=1)
+    async def send(self, logger: Optional[object] = None):
+        @with_retries(success_max=1, logger=logger)
         async def __send():
             await self.__batch.send()
+            if logger:
+                batch_parts = self.__batch._batch
+                for batch_part_key, batch_part in batch_parts.items():
+                    logger.info(f'batch partition {batch_part_key} BEGIN')
+                    for clustering, batch_part_clustering in batch_part.batch.items():
+                        logger.info(f'batch partition {batch_part_key} clustering {clustering} value {batch_part_clustering.new_value} has been sent')
+                    logger.info(f'batch partition {batch_part_key} END')
 
         for entry in self.__cas_keys:
             entry_cas = await KVClient.get_entry_with_version(self.__session, **entry)
@@ -224,11 +236,11 @@ class KVClient:
             raise BatchDoesNotExist()
 
     @staticmethod
-    async def execute_batch(b_id: int):
+    async def execute_batch(b_id: int, logger: Optional[object] = None):
         batch = KVClient.get_batch(b_id)
 
         try:
-            await batch.send()
+            await batch.send(logger)
         except Exception:
             raise BatchSendError()
         finally:
@@ -243,7 +255,7 @@ class KVClient:
             await self._batch.send()
         except Exception as e:
             if self.logger:
-                self.logger.error(e)
+                self.logger.exception(e)
 
     @staticmethod
     async def listen(session: Session, partition_key: List[str], clustering_key: Optional[List[str]],
